<?php
header('Content-Type: application/json');
ini_set('display_errors', 1);
error_reporting(0);

$env = parse_ini_file(__DIR__ . '/variables.env');
$db_password = $env['DB_PASS'];
$api_key = $env['KEY'];

$option = isset($_GET['option']) ? $_GET['option'] : '';

$conn = new mysqli("devweb2024.cis.strath.ac.uk", "wyb20122", "$db_password", "wyb20122");

$provided_key = isset($_GET['key'])
    ? $_GET['key']
    : (isset($_POST['key'])
        ? $_POST['key']
        : '');

if ($provided_key !== $api_key) {
    echo json_encode(["error" => "Invalid API key"]);
    exit();
}

if ($conn->connect_error) {
    echo json_encode(["error" => "Database connection failed"]);
    exit();
}

switch ($option) {
    case 'getUser':

        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';

        if (!$userID) {
            echo json_encode(["error" => "Missing userID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT age, sex, weight FROM USERS WHERE userID = ?");
        $stmt->bind_param("s", $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = $result->fetch_assoc();

        echo json_encode($data ?: ["error" => "User not found"]);
        $stmt->close();
        break;

    case 'getForums':

        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';

        if (!$userID) {
            echo json_encode(["error" => "Missing userID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM FORUMS WHERE forumID NOT IN(SELECT * FROM FLAGGED WHERE userID = ?);[*]");
        $stmt->bind_param("s", $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = $result->fetch_assoc();

        echo json_encode($data ?: ["error" => "No Forums"]);
        $stmt->close();
        break;

    case 'getForum':
        $forumID = isset($_GET['forumID']) ? $_GET['forumID'] : '';

        if (!$forumID) {
            echo json_encode(["error" => "Missing forumID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM FORUMS WHERE forumID = ?");
        $stmt->bind_param("i", $forumID);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = $result->fetch_assoc();

        echo json_encode($data ?: ["error" => "Forum not found"]);
        $stmt->close();
        break;

    case 'getReplies':

        $forumID = isset($_GET['forumID']) ? $_GET['forumID'] : '';
        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';

        if (!$forumID || !$userID) {
            echo json_encode(["error" => "Missing forumID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM REPLIES WHERE forumID = ? AND replyID NOT IN(SELECT * FROM FLAGGED WHERE userID = ?);[**]");
        $stmt->bind_param("s", $forumID, $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = $result->fetch_assoc();

        echo json_encode($data ?: ["error" => "forum not found"]);
        $stmt->close();
        break;

    case 'flagForumReply':
        $replyID = isset($_POST['replyID']) ? $_POST['replyID'] : null;
        $userID = isset($_POST['userID']) ? $_POST['userID'] : null;
        $reason = isset($_POST['reason']) ? $_POST['reason'] : '';

        if (!$replyID || !$userID || !$reason) {
            echo json_encode(["error" => "Missing parameters"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO FLAGGED (replyID, userID, reason, status) VALUES (?, ?, ?, 'requested')");
        $stmt->bind_param("iss", $replyID, $userID, $reason);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;


    case 'flagForum':
        $forumID = isset($_POST['forumID']) ? $_POST['forumID'] : null;
        $userID = isset($_POST['userID']) ? $_POST['userID'] : null;
        $reason = isset($_POST['reason']) ? $_POST['reason'] : '';

        if (!$forumID || !$userID || !$reason) {
            echo json_encode(["error" => "Missing parameters"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO FLAGGED (forumID, userID, reason, status) VALUES (?, ?, ?, 'requested')");
        $stmt->bind_param("iss", $forumID, $userID, $reason);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'postForum':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $topic = isset($_POST['topic']) ? $_POST['topic'] : '';
        $body = isset($_POST['body']) ? $_POST['body'] : '';

        if (!$userID || !$topic || !$body) {
            echo json_encode(["error" => "Missing fields"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO FORUMS (userID, topic, body) VALUES (?, ?, ?)");
        $stmt->bind_param("sss", $userID, $topic, $body);
        $stmt->execute();

        echo json_encode(["success" => true, "forumID" => $stmt->insert_id]);
        $stmt->close();
        break;

    case 'deleteForum':
        $forumID = isset($_POST['forumID']) ? $_POST['forumID'] : '';

        if (!$forumID) {
            echo json_encode(["error" => "Missing forumID"]);
            break;
        }

        $stmt = $conn->prepare("DELETE FROM FORUMS WHERE forumID = ?");
        $stmt->bind_param("i", $forumID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'replyToForum':
        $forumID = isset($_POST['forumID']) ? $_POST['forumID'] : '';
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $body = isset($_POST['body']) ? $_POST['body'] : '';

        if (!$forumID || !$userID || !$body) {
            echo json_encode(["error" => "Missing fields"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO REPLIES (forumID, userID, body) VALUES (?, ?, ?)");
        $stmt->bind_param("iss", $forumID, $userID, $body);
        $stmt->execute();

        echo json_encode(["success" => true, "replyID" => $stmt->insert_id]);
        $stmt->close();
        break;

    case 'upvote':
        $forumID = isset($_POST['forumID']) ? $_POST['forumID'] : '';
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $increment = isset($_POST['increment']) ? $_POST['increment'] : null;

        if (!$forumID || !$userID || !isset($increment)) {
            echo json_encode(["error" => "Missing vote data"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO VOTES (forumID, userID, increment) VALUES (?, ?, ?)");
        $stmt->bind_param("isi", $forumID, $userID, $increment);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'totalVotes':
        $forumID = isset($_GET['forumID']) ? $_GET['forumID'] : '';

        if (!$forumID) {
            echo json_encode(["error" => "Missing forumID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT SUM(increment) AS votes FROM VOTES WHERE forumID = ?");
        $stmt->bind_param("i", $forumID);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = $result->fetch_assoc();

        echo json_encode($data ?: ["votes" => 0]);
        $stmt->close();
        break;

    case 'editUpvote':
        $forumID = isset($_POST['forumID']) ? $_POST['forumID'] : '';
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $increment = isset($_POST['increment']) ? $_POST['increment'] : null;

        if (!$forumID || !$userID || !isset($increment)) {
            echo json_encode(["error" => "Missing vote data"]);
            break;
        }

        $stmt = $conn->prepare("UPDATE VOTES SET increment = ? WHERE forumID = ? AND userID = ?");
        $stmt->bind_param("iis", $increment, $forumID, $userID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'getMessages':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';
        $recipient = isset($_GET['recipient']) ? $_GET['recipient'] : '';

        if (!$username || !$recipient) {
            echo json_encode(["error" => "Missing userID or recipient"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM MESSAGES WHERE (sender = ? AND recipient = ?) OR (sender = ? AND recipient = ?) ORDER BY time ASC");
        $stmt->bind_param("ssss", $username, $recipient, $recipient, $username);
        $stmt->execute();
        $result = $stmt->get_result();
        $data = [];

        while ($row = $result->fetch_assoc()) {
            $data[] = $row;
        }

        echo json_encode($data);
        $stmt->close();
        break;

    case 'sendMessage':
        $sender = isset($_POST['userID']) ? $_POST['userID'] : '';
        $recipient = isset($_POST['recipient']) ? $_POST['recipient'] : '';
        $body = isset($_POST['body']) ? $_POST['body'] : '';

        if (!$sender || !$recipient || !$body) {
            echo json_encode(["error" => "Missing message fields"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO MESSAGES (sender, recipient, body) VALUES (?, ?, ?)");
        $stmt->bind_param("sss", $sender, $recipient, $body);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'updateUserProfile':
        $username = isset($_POST['userID']) ? $_POST['userID'] : '';
        $forename = isset($_POST['forename']) ? $_POST['forename'] : '';
        $surname = isset($_POST['surname']) ? $_POST['surname'] : '';
        $age = isset($_POST['age']) ? $_POST['age'] : null;
        $sex = isset($_POST['sex']) ? $_POST['sex'] : null;
        $weight = isset($_POST['weight']) ? $_POST['weight'] : null;
        $height = isset($_POST['height']) ? $_POST['height'] : null;
        $visibility = isset($_POST['visibility']) ? $_POST['visibility'] : 'Private';

        $stmt = $conn->prepare("UPDATE USERS SET forename = ?, surname = ?, age = ?, sex = ?, weight = ?, height = ?, visibility = ? WHERE userID = ?");
        $stmt->bind_param("ssiisiis", $forename, $surname, $age, $sex, $weight, $height, $visibility, $username);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'getNearbyUsers':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';
        $lat = isset($_GET['lat']) ? $_GET['lat'] : '';
        $lon = isset($_GET['lon']) ? $_GET['lon'] : '';

        if (!$username || !$lat || !$lon) {
            echo json_encode(["error" => "Missing coordinates or userID"]);
            break;
        }

        $stmt = $conn->prepare("
            SELECT userID FROM USERS 
            WHERE userID != ? AND visibility = 'Public' AND 
            (6371 * acos(cos(radians(?)) * cos(radians(lat)) * cos(radians(lon) - radians(?)) + sin(radians(?)) * sin(radians(lat)))) <= 10
        ");
        $stmt->bind_param("sddd", $username, $lat, $lon, $lat);
        $stmt->execute();
        $result = $stmt->get_result();
        $users = [];

        while ($row = $result->fetch_assoc()) {
            $users[] = $row;
        }

        echo json_encode($users);
        $stmt->close();
        break;

    case 'getPublicUsers':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';

        $stmt = $conn->prepare("SELECT userID FROM USERS WHERE userID != ? AND visibility = 'Public'");
        $stmt->bind_param("s", $username);
        $stmt->execute();
        $result = $stmt->get_result();
        $users = [];

        while ($row = $result->fetch_assoc()) {
            $users[] = $row;
        }

        echo json_encode($users);
        $stmt->close();
        break;

    case 'getRequests':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';

        $stmt = $conn->prepare("SELECT * FROM CONNECTIONS WHERE recipient = ? AND status = 'pending'");
        $stmt->bind_param("s", $username);
        $stmt->execute();
        $result = $stmt->get_result();
        $requests = [];

        while ($row = $result->fetch_assoc()) {
            $requests[] = $row;
        }

        echo json_encode($requests);
        $stmt->close();
        break;

    case 'approveRequest':
        $requestID = isset($_POST['requestID']) ? $_POST['requestID'] : '';

        $stmt = $conn->prepare("UPDATE CONNECTIONS SET status = 'approved' WHERE requestID = ?");
        $stmt->bind_param("i", $requestID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'declineRequest':
        $requestID = isset($_POST['requestID']) ? $_POST['requestID'] : '';

        $stmt = $conn->prepare("DELETE FROM CONNECTIONS WHERE requestID = ?");
        $stmt->bind_param("i", $requestID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'getApprovedConnections':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';

        $stmt = $conn->prepare("SELECT * FROM CONNECTIONS WHERE (sender = ? OR recipient = ?) AND status = 'approved' ORDER BY requestID DESC");
        $stmt->bind_param("ss", $username, $username);
        $stmt->execute();
        $result = $stmt->get_result();
        $connections = [];

        while ($row = $result->fetch_assoc()) {
            $connections[] = $row;
        }

        echo json_encode($connections);
        $stmt->close();
        break;

    case 'updateBio':
        $username = isset($_POST['userID']) ? $_POST['userID'] : '';
        $bio = isset($_POST['bio']) ? $_POST['bio'] : '';

        $stmt = $conn->prepare("UPDATE USERS SET bio = ? WHERE userID = ?");
        $stmt->bind_param("ss", $bio, $username);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'updateImage':
        $username = isset($_POST['userID']) ? $_POST['userID'] : '';
        $img = isset($_POST['image']) ? $_POST['image'] : null;

        $stmt = $conn->prepare("UPDATE USERS SET image = ? WHERE userID = ?");
        $stmt->bind_param("bs", $img, $username); // assumes image is sent as blob-safe base64-encoded data
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'getRoutes':
        $username = isset($_GET['userID']) ? $_GET['userID'] : '';

        $stmt = $conn->prepare("SELECT * FROM ROUTES WHERE userID = ? AND visibility = 'public'");
        $stmt->bind_param("s", $username);
        $stmt->execute();
        $result = $stmt->get_result();
        $routes = [];

        while ($row = $result->fetch_assoc()) {
            $routes[] = $row;
        }

        echo json_encode($routes);
        $stmt->close();
        break;

    case 'getUserRoutes':
        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';

        $stmt = $conn->prepare("
            SELECT r.routeID, r.userID, r.xml, r.visibility, ur.pb, ur.favourite
            FROM ROUTES r
            LEFT JOIN USER_ROUTES ur ON r.routeID = ur.routeID AND ur.userID = ?
            WHERE r.userID = ? OR ur.favourite = 1
        ");
        $stmt->bind_param("ss", $userID, $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $routes = [];

        while ($row = $result->fetch_assoc()) {
            $routes[] = $row;
        }

        echo json_encode($routes);
        $stmt->close();
        break;

    case 'updateFavouriteRoute':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $routeID = isset($_POST['routeID']) ? $_POST['routeID'] : '';
        $favourite = isset($_POST['favourite']) ? $_POST['favourite'] : 0;

        $stmt = $conn->prepare("
            INSERT INTO USER_ROUTES (userID, routeID, favourite) 
            VALUES (?, ?, ?)
            ON DUPLICATE KEY UPDATE FAVOURITE = ?
        ");
        $stmt->bind_param("siii", $userID, $routeID, $favourite, $favourite);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'updateRouteVisibility':
        $routeID = isset($_POST['routeID']) ? $_POST['routeID'] : '';
        $visibility = isset($_POST['visibility']) ? $_POST['visibility'] : 'Private';

        $stmt = $conn->prepare("UPDATE ROUTES SET visibility = ? WHERE routeID = ?");
        $stmt->bind_param("si", $visibility, $routeID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'login':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $password = isset($_POST['password']) ? $_POST['password'] : '';

        if (!$userID || !$password) {
            echo json_encode(["error" => "Missing login credentials"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM USERS WHERE userID = ? AND password = ?");
        $stmt->bind_param("ss", $userID, $password);
        $stmt->execute();
        $result = $stmt->get_result();
        $user = $result->fetch_assoc();

        echo json_encode($user ?: ["error" => "Invalid login"]);
        $stmt->close();
        break;

    case 'signup':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $password = isset($_POST['password']) ? $_POST['password'] : '';
        $forename = isset($_POST['forename']) ? $_POST['forename'] : '';
        $surname = isset($_POST['surname']) ? $_POST['surname'] : '';
        $age = isset($_POST['age']) ? $_POST['age'] : null;
        $sex = isset($_POST['sex']) ? $_POST['sex'] : null;
        $weight = isset($_POST['weight']) ? $_POST['weight'] : null;
        $height = isset($_POST['height']) ? $_POST['height'] : null;
        $visibility = isset($_POST['visibility']) ? $_POST['visibility'] : 'Private';

        if (!$userID || !$password || !$forename || !$surname) {
            echo json_encode(["error" => "Missing required fields"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO USERS (userID, password, forename, surname, age, sex, weight, height, visibility) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
        $stmt->bind_param("ssssissis", $userID, $password, $forename, $surname, $age, $sex, $weight, $height, $visibility);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'updatePassword':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $newPassword = isset($_POST['newPassword']) ? $_POST['newPassword'] : '';

        if (!$userID || !$newPassword) {
            echo json_encode(["error" => "Missing password change fields"]);
            break;
        }

        $stmt = $conn->prepare("UPDATE USERS SET password = ? WHERE userID = ?");
        $stmt->bind_param("ss", $newPassword, $userID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'getProfile':
        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';

        if (!$userID) {
            echo json_encode(["error" => "Missing userID"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM USERS WHERE userID = ?");
        $stmt->bind_param("s", $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $user = $result->fetch_assoc();

        echo json_encode($user ?: ["error" => "User not found"]);
        $stmt->close();
        break;

    case 'checkConnection':
        $userID = isset($_GET['userID']) ? $_GET['userID'] : '';
        $otherUserID = isset($_GET['otherUserID']) ? $_GET['otherUserID'] : '';

        if (!$userID || !$otherUserID) {
            echo json_encode(["error" => "Missing user IDs"]);
            break;
        }

        $stmt = $conn->prepare("SELECT * FROM CONNECTIONS WHERE (sender = ? AND recipient = ?) OR (sender = ? AND recipient = ?)");
        $stmt->bind_param("ssss", $userID, $otherUserID, $otherUserID, $userID);
        $stmt->execute();
        $result = $stmt->get_result();
        $connData = $result->fetch_assoc();

        echo json_encode($connData ?: ["status" => "none"]);
        $stmt->close();
        break;

    case 'sendFriendRequest':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $recipient = isset($_POST['recipient']) ? $_POST['recipient'] : '';

        if (!$userID || !$recipient) {
            echo json_encode(["error" => "Missing request fields"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO CONNECTIONS (sender, recipient, status) VALUES (?, ?, 'requested')");
        $stmt->bind_param("ss", $userID, $recipient);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'uploadRoute':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $xml = isset($_POST['xml']) ? $_POST['xml'] : '';
        $visibility = isset($_POST['visibility']) ? $_POST['visibility'] : 'Private';

        if (!$userID || !$xml) {
            echo json_encode(["error" => "Missing route data"]);
            break;
        }

        $stmt = $conn->prepare("INSERT INTO ROUTES (userID, xml, visibility) VALUES (?, ?, ?)");
        $stmt->bind_param("sss", $userID, $xml, $visibility);
        $stmt->execute();

        echo json_encode(["success" => true, "routeID" => $stmt->insert_id]);
        $stmt->close();
        break;

    case 'updateLocation':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';
        $lat = isset($_POST['lat']) ? $_POST['lat'] : null;
        $lon = isset($_POST['lon']) ? $_POST['lon'] : null;

        if (!$userID || $lat === null || $lon === null) {
            echo json_encode(["error" => "Missing location data"]);
            break;
        }

        $stmt = $conn->prepare("UPDATE USERS SET lat = ?, lon = ? WHERE userID = ?");
        $stmt->bind_param("dds", $lat, $lon, $userID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'clearVotes':
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';

        if (!$userID) {
            echo json_encode(["error" => "Missing userID"]);
            break;
        }

        $stmt = $conn->prepare("DELETE FROM VOTES WHERE userID = ?");
        $stmt->bind_param("s", $userID);
        $stmt->execute();

        echo json_encode(["success" => true]);
        $stmt->close();
        break;

    case 'deleteRouteAll':
        $routeID = isset($_POST['routeID']) ? $_POST['routeID'] : '';
        $userID = isset($_POST['userID']) ? $_POST['userID'] : '';

        if (!$routeID || !$userID) {
            echo json_encode(["error" => "Missing routeID or userID"]);
            break;
        }

        $stmt1 = $conn->prepare("DELETE FROM USER_ROUTES WHERE routeID = ? AND userID = ?");
        $stmt1->bind_param("is", $routeID, $userID);
        $stmt1->execute();
        $stmt1->close();

        $stmt2 = $conn->prepare("DELETE FROM ROUTES WHERE routeID = ? AND userID = ?");
        $stmt2->bind_param("is", $routeID, $userID);
        $stmt2->execute();
        $stmt2->close();

        echo json_encode(["success" => true]);
        break;




    default:
        echo json_encode(["error" => "Invalid option"]);
        break;
}

$conn->close();
?>
